import sudoku.PuzzleBox;
import sudoku.Background;
import sudoku.Cell;
import sudoku.Grid;
import sudoku.Headline;
import sudoku.RRButton;
import sudoku.PuzzleLoader;
import sudoku.LittleProgressBar;
import sudoku.Random;	// my own generator
import sudoku.DiffSlider;	// my own slider
import sudoku.OptionsPanel;	// my own slider

stop();


// to do
//	√ - new puzzle button
//	√ - progress bar during loading
//	√ - some response when the puzzle is finished correctly
//	√ - timer
//	√ - pause button
//	√ - puzzle permutations, to provide variety
//	√ - puzzle-level selection
//	√ - new puzzle button
//	√ - put on www.txbobsc.com and homepage.mac.com/bobsc
//	√ - Show number remaining after Check button used.
//	√ - get puzzles from www.dailysudoku.com and websudoku.com
//	√ - display puzzle level indication
//  √ - do some evaluation, and display the results, when Check button is clicked
//	√ - use PuzzleBox class to hold the cells
//	o - timeout on loading
//	o - "Harden" button
//	o - "Options..." button
//	o - sound effects


// Start loading a new puzzle
var rand:Random = new Random();
rand.randomize();
var puzzleSource:uint = 1; // 1=XML, 2=JavaScript, 3=External Sites
var puzzleLevel:uint = rand.next()*4+1; // 1...4 == Easy, Medium, Hard, Very Hard
// If puzzleSource == 3 then puzzleLevel == 5 means get puzzle from getit.php
// If puzzleSource == 3 then puzzleLevel == 6 means get puzzle from getUSAT.php

var optionsPanel:OptionsPanel = new OptionsPanel(50, 50, 446, 446);
if (optionsPanel.option1) {
	puzzleSource = 3;	// get puzzle from external source
	puzzleLevel = rand.next()*5;	// 0...4
}
var findSingletons:Boolean = optionsPanel.option2;

var progBar:LittleProgressBar = new LittleProgressBar(175, 10, 200, 5);
addChild(progBar);
var html:PuzzleLoader = new PuzzleLoader(progBar, puzzleSource, puzzleLevel);

var cheat:String = "684251379193687245725943816432598167816734592957126483361875924579412638248369751"; 
var masks:String = "001100001011101000001010011000100100100000001001001000110010100000101110100001100";

var startTime:Date;
var savedTime:Number = 0;	// millisconds used before pause

const puzzleLeft:int = 50;
const puzzleTop:int = 50;
const puzzleWidth:int = 450;
const cellSize:int = 50;

// Banner
var banner:String;
var bannerTimer:Timer;
var finishedMsg:Headline = new Headline(550);
addChild(finishedMsg);

// Puzzle
addChild(new Background(puzzleLeft, puzzleTop, puzzleWidth));
var puzzleBox:PuzzleBox = new PuzzleBox(puzzleLeft, puzzleTop, cellSize, cellSize);
addChild(puzzleBox);
var grid:Grid = new Grid(puzzleLeft, puzzleWidth);
addChild(grid);	// add the grid on top of the cells

// Controls
var checkButton:RRButton = new RRButton(50, 505, 60, "Check", onCheckBtn);
addChild(checkButton);
var hintsButton:RRButton = new RRButton(50, 528, 60, "Hints", onHintsBtn);
addChild(hintsButton);
var newButton:RRButton = new RRButton(200, 514, 75, "New Puzzle", onNewBtn);
addChild(newButton);
var diffSlider = new DiffSlider(285, 505);
addChild(diffSlider);
var resetButton:RRButton = new RRButton(440, 528, 60, "Reset", onResetBtn);
addChild(resetButton);
var pauseButton:RRButton = new RRButton(440, 505, 60, "Pause", onPauseBtn);
addChild(pauseButton);
var resumeButton:RRButton = new RRButton(440, 505, 60, "Resume", onResumeBtn);

addChild(optionsPanel);

if (html.ready) {
	runPuzzle(null);
} else {
	html.addEventListener(html.HTML_READY_EVENT, runPuzzle);
}

// =============================================
//	Event Handlers
// =============================================
function runPuzzle(event:Event):void
{
	removeChild(progBar);
	progBar = null;
	if (html.ready) {
		cheat = html.cheat;
		masks = html.masks;
	}
	cheat = permutePuzzleDigits(cheat);
	if (rand.next() > .5) {
		cheat = transposePuzzle(cheat);
		masks = transposePuzzle(masks);
	}

	puzzleBox.Populate(cheat, masks);
	
	checkButton.enabled = true;
	hintsButton.enabled = true;
	newButton.enabled = true;
	pauseButton.enabled = true;
	resetButton.enabled = true;
	
	startTime = new Date();
	finishedMsg.text = "Play Sudoku!!! -- Level: " + html.getDifficulty();

	diffSlider.value = html.difficulty;
}

function onCheckBtn(evt:Event):void
{
	fixButtonsAndBanner();
	var endTime:Date = new Date();
	var solveTime:int = Math.round((endTime.time - startTime.time + savedTime) / 1000);
	var solveMinutes:int = solveTime / 60;
	var solveSeconds:int = solveTime % 60;
	var numCorrect:uint = puzzleBox.Check();
	if (numCorrect == 81) {
		// Finished!!
		// disable all buttons except New and Reset
		checkButton.enabled = false;
		hintsButton.enabled = false;
		pauseButton.enabled = false;

		finishedMsg.text = "Congratulations! You took " + solveMinutes + " minutes and " + solveSeconds + " seconds.";
		
		// try to write new average on user's computer
		try {
			var solName:String = "average" + html.getDifficulty();
			var pat:RegExp = /[ ]/g;
			var lso:SharedObject = SharedObject.getLocal(solName.replace(pat, "_"));
			var num:uint = 0;
			var ave:uint = 0;
			if (lso.data.number) {
				num = lso.data.number;
				ave = lso.data.average;
			}
			ave = (num*ave + solveTime) / (num + 1);
			num++;
			lso.data.number = num;
			lso.data.average = ave;
			var flushResult:String = lso.flush();
			if (num > 1) {
				banner = "Your average for " + num + " ";
				banner += html.getDifficulty().toLowerCase() + " puzzles is ";
				solveMinutes = ave / 60;
				solveSeconds = ave % 60;
				if (solveMinutes > 0) {
					banner += solveMinutes + " minutes ";
				}
				banner += solveSeconds + " seconds.";
				if (bannerTimer == null) {
					bannerTimer = new Timer(2000);
					bannerTimer.addEventListener(TimerEvent.TIMER, onRotateBanner);
				}
				bannerTimer.start();
			}
		}
		catch(e:Error) {
			// Security.showSettings(SecurityPanel.LOCAL_STORAGE);
		}
	} else {
		finishedMsg.text = (81-numCorrect) + " to go; time so far is " + solveMinutes + " minutes and " + solveSeconds + " seconds.";
	}
}

function onRotateBanner(event:TimerEvent):void
{
	var saveBanner:String = finishedMsg.text;
	finishedMsg.text = banner;
	banner = saveBanner;
}

function onHintsBtn(evt:Event):void
{
	fixButtonsAndBanner();
	puzzleBox.findPossibles();
	if (optionsPanel.option2) {
		puzzleBox.findSingletons();
	}
}

function onPauseBtn(evt:Event):void
{
	// exchange Pause and Resume buttons
	addChild(resumeButton);
	resumeButton.enabled = true;
	removeChild(pauseButton);
	checkButton.enabled = false;
	hintsButton.enabled = false;
	newButton.enabled = false;
	resetButton.enabled = false;
	
	// save the time
	var pauseTime:Date = new Date();
	savedTime += pauseTime.time - startTime.time;

	// hide the puzzle
	puzzleBox.Hide();
	
	optionsPanel.Show();
}

function onResumeBtn(evt:Event=null):void
{
	optionsPanel.Hide();
	// exchange Pause and Resume buttons
	addChild(pauseButton);
	removeChild(resumeButton);
	checkButton.enabled = true;
	hintsButton.enabled = true;
	newButton.enabled = true;
	resetButton.enabled = true;
	
	// show the puzzle
	puzzleBox.Show();
	
	// start the timer
	startTime = new Date();
}

function onResetBtn(evt:Event):void
{
	fixButtonsAndBanner();
	puzzleBox.Reset();
	checkButton.enabled = true;
	hintsButton.enabled = true;
	pauseButton.enabled = true;
}

function onNewBtn(evt:Event):void
{
	fixButtonsAndBanner();

	checkButton.enabled = false;
	hintsButton.enabled = false;
	pauseButton.enabled = false;
	newButton.enabled = false;
	resetButton.enabled = false;

	// which level is selected?
	if (optionsPanel.option1) {
		puzzleSource = 3;	// get puzzle from external source
		puzzleLevel = diffSlider.value;
	} else {
		puzzleSource = 1;	// get puzzle from my xml file
		puzzleLevel = diffSlider.value;
	}
	finishedMsg.text = "";
	savedTime = 0;
	progBar = new LittleProgressBar(175, 10, 200, 5);
	addChild(progBar);
	html = new PuzzleLoader(progBar, puzzleSource, puzzleLevel);
	html.addEventListener(html.HTML_READY_EVENT, runPuzzle);
}

function fixButtonsAndBanner()
{
	if (bannerTimer) bannerTimer.stop();
}


// =============================================
//	Functions
// =============================================

function permutePuzzleDigits(puzzle:String):String
{
	// given an 81 character string of digits, or digits and periods
	// randomly permute the digits 1...9
	if (puzzle.length != 81) {
		return puzzle;
	}
	// permute digits 1...9
	var digits:Array = [ 1,2,3,4,5,6,7,8,9 ];
	var i:int;
	for (i=0; i<20; i++) {
		var j:int = rand.next()*9;
		var k:int = rand.next()*9;
		var digit:int = digits[j];
		digits[j] = digits[k];
		digits[k] = digit;
	}
	// build permuted puzzle
	var newpuzzle:String = "";
	for (i=0; i<81; i++) {
		var ch:int = puzzle.charCodeAt(i) - 0x30;
		if ((ch >= 1) && (ch <= 9)) {
			ch = digits[ch-1];
		}
		newpuzzle += String.fromCharCode(ch + 0x30);
	}
	return newpuzzle;
}

function transposePuzzle(puzzle:String):String{
	var newpuzzle:String = "";
	var i:int, j:int;
	for (i=0; i<9; i++) {
		for (j=0; j<9; j++) {
			newpuzzle += puzzle.charAt(j*9+i);
		}
	}
//trace(puzzle);
//trace(newpuzzle);
	return newpuzzle;
}
